
/*!
	@file	CCell.h
	@brief	cell class
*/

#ifndef __H_CCELL__
#define __H_CCELL__

#include <string>
#include <vector>
#include "float3.h"
#include "COutline.h"

using namespace std;

namespace ddv {

class CCell: public float3 {
public:
	CCell() {};
	CCell( int id, int parent_id, string name, float3 f3pos, int time )
		: float3( f3pos ), t( time )
		, m_nID( id ), m_nParentID( parent_id ), m_sName( name ) {};
	CCell( int id, int parent_id, string name
		, float fx, float fy, float fz, int time )
		: float3( fx, fy, fz ), t( time )
		, m_nID( id ), m_nParentID( parent_id ), m_sName( name ) {};
	~CCell() {};

	void clear() {
		x = 0;
		y = 0;
		z = 0;
		t = 0;
		m_nID = 0;
		m_nParentID = 0;
		m_sName.clear();
		m_outlines.clear();
		m_daughters.clear();
	};

	void operator =( CCell cell ) {
		t = cell.t;
		m_nID = cell.m_nID;
		m_nParentID = cell.m_nParentID;
		m_sName = cell.m_sName;
		m_outlines = cell.m_outlines;
		m_daughters = cell.m_daughters;
	};

	void operator =( float3 f3 ) {
		float3::operator =( f3 );
	};

	inline int getID() {
		return m_nID;
	};
	inline int getParentID() {
		return m_nParentID;
	};
	inline string getName() {
		return m_sName;
	};
	inline vector< COutline > *getOutlines() {
		return &m_outlines;
	};
	inline vector< int > *getDaughters() {
		return &m_daughters;
	};

	int t;

protected:
	int m_nID;
	int m_nParentID;
	string m_sName;
	vector< COutline > m_outlines;
	vector< int > m_daughters;
};

} // namespace ddv

#endif // __H_CCELL__

