
/*!
	@file	CCell_io.h
	@brief	cell input/output class
*/

#ifndef __H_CCELL_IO__
#define __H_CCELL_IO__

#include <string>
#include <vector>
#include "CCell.h"
#include "int3.h"
#include "float3.h"

using namespace std;

namespace ddv {

class CCell_io: public CCell {
public:
	CCell_io() {};
	CCell_io( int id, int parent_id, string name, float3 f3pos, int time )
		: CCell( id, parent_id, name, f3pos, time ) {};
	CCell_io( int id, int parent_id, string name
		, float fx, float fy, float fz, int time )
		: CCell( id, parent_id, name, fx, fy, fz, time ) {};
	~CCell_io() {};

	inline void set( int id, int parent_id, string name
		, float3 f3pos, int time ) {
		CCell::operator =( f3pos );
		t = time;
		m_nID = id;
		m_nParentID = parent_id;
		m_sName = name;
	};
	inline void set( int id, int parent_id, string name
		, float fx, float fy, float fz, int time ) {
		set( id, parent_id, name, float3( fx, fy, fz ), time );
	};
	inline void setName( string sName ) {
		m_sName = sName;
	};

	inline void addDaughter( int nID ) {
		m_daughters.push_back( nID );
	};

	bool appendOutline( int3 n3pos, string sOutline );
	inline bool appendOutline( int x, int y, int z, string sOutline ) {
		return appendOutline( int3( x, y, z ), sOutline );
	};

protected:
};

} // namespace ddv

#endif // __H_CCELL__

