
/*!
	@file	CDataset.h
	@brief	dataset class
*/

#ifndef __H_CDATASET__
#define __H_CDATASET__

#include <list>
#include <vector>
#include "float3.h"
#include "CCell.h"

using namespace std;

namespace ddv {

class CDataset {
public:
	CDataset(): m_bLineageExists( false ), m_bCellNameExists( false )
		, m_bROIExists( false ) {
		clear();
	};

	void clear() {
		m_bLineageExists = false;
		m_bCellNameExists = false;
		m_bROIExists = false;
		m_cells.clear();
		m_indexbyTimePoint.clear();
		m_fAverageFocalPlane = 0.0f;
	};

	void operator =( CDataset dataset ) {
		m_bLineageExists = dataset.m_bLineageExists;
		m_bCellNameExists = dataset.m_bCellNameExists;
		m_bROIExists = dataset.m_bROIExists;
		m_cells = dataset.m_cells;
		m_indexbyTimePoint.clear();
		vector< CCell * > v;
		for ( unsigned int i = 0; i < dataset.m_indexbyTimePoint.size()
			; ++ i ) {
			m_indexbyTimePoint.push_back( v );
			for ( unsigned int j = 0; j < dataset.m_indexbyTimePoint[ i ].size()
				; ++ j ) {
				m_indexbyTimePoint[ i ].push_back( getCell(
					( dataset.m_indexbyTimePoint[ i ][ j ] )->getID() ) );
			}
		}
		m_fAverageFocalPlane = dataset.m_fAverageFocalPlane;
	};

	inline bool isLineageExists() {
		return m_bLineageExists;
	};
	inline bool isCellNameExists() {
		return m_bCellNameExists;
	};
	inline bool isROIExists() {
		return m_bROIExists;
	};

	inline int getMinTimePoint() {
	  	return 1;
	};

	inline int getMaxTimePoint() {
		return m_indexbyTimePoint.size();
	};

	inline float getAverageFocalPlane() {
		return m_fAverageFocalPlane;
	};

	inline CCell *getCell( int nID ) {
		list< CCell >::iterator it = m_cells.begin();
		for ( ; it != m_cells.end(); ++ it ) {
			if ( it->getID() == nID ) {
				return &*it;
			}
		}
		return NULL;
	};

	inline vector< CCell * > *getCellAt( int nTimePoint ) {
		if ( !nTimePoint
			|| ( unsigned int ) nTimePoint > m_indexbyTimePoint.size() ) {
			return &emptyCellVector;
		}
		return &m_indexbyTimePoint[ nTimePoint - 1 ];
	};

protected:
	bool m_bLineageExists;
	bool m_bCellNameExists;
	bool m_bROIExists;

	list< CCell > m_cells;
	vector< vector< CCell * > > m_indexbyTimePoint;
	float m_fAverageFocalPlane;
	static vector< CCell * > emptyCellVector;
};

} // namespace ddv

#endif // __H_CDATASET__

