
/*!
	@file	float3.h
	@brief	float array
*/

#ifndef __H_FLOAT3__
#define __H_FLOAT3__

#include <math.h>

namespace ddv {

typedef struct float3 {
public:
	float3(): x( 0.0f ), y( 0.0f ), z( 0.0f ) {};
	float3( float fx, float fy, float fz ): x( fx ), y( fy ), z( fz ) {};

	inline void operator = ( float3 f3 ) {
		x = f3.x;
		y = f3.y;
		z = f3.z;
	};

	inline float3 operator + ( float3 f ) {
		return float3( x + f.x, y + f.y, z + f.z );
	};
	inline void operator += ( float3 f ) {
		x += f.x;
		y += f.y;
		z += f.z;
	};
	inline float3 operator - ( float3 f ) {
		return float3( x - f.x, y - f.y, z - f.z );
	};
	inline void operator -= ( float3 f ) {
		x -= f.x;
		y -= f.y;
		z -= f.z;
	};
	inline float3 operator * ( float &f ) {
		return float3( x * f, y * f, z * f );
	};
	inline float operator * ( float3 &f3 ) {
		return dot( f3 );
	};
	inline float3 operator / ( float f ) {
		return float3( x / f, y / f, z / f );
	};
	inline float3 operator - () {
		return float3( -x, -y, -z );
	};

	inline float dot( float3 &f3 ) {
		return x * f3.x + y * f3.y + z * f3.z;
	};
	inline float3 cross( float3 &f3 ) {
		return float3( y * f3.z - z * f3.y
			, z * f3.x - x * f3.z
			, x * f3.y - y * f3.x );
	};
	inline float abs() {
		return sqrt( x * x + y * y + z * z );
	};
	inline float3 normalize() {
		float d = abs();
		if ( d ) {
			return float3( x / d, y / d, z / d );
		}
		return float3( 0.0f, 0.0f, 0.0f );
	};

	float x;
	float y;
	float z;
} float3;

} // ddv

#endif // __H_FLOAT3__

