
#include <string>
#include <QDir>
#include <QtGui>
#include <QFileDialog>
#include <QtXml>
#include <iostream>
#include "frmDevDynViewer.h"
#include "CQDataset_io.h"

namespace ddv {

void frmDevDynViewer::slotActionFloatControlPanel_toggled( bool bOnOff ) {
	m_bControlPanelSwitch = !bOnOff;
	if ( m_bControlPanelSwitch ) {
		floatControlPanelToStaticControlPanel();
	} else {
		staticControlPanelToFloatControlPanel();
	}
}

void frmDevDynViewer::slotControlPanel_switched() {
	actionFloatControlpanel->setChecked( m_bControlPanelSwitch );
}

void frmDevDynViewer::staticControlPanelToFloatControlPanel() {
	frameScrollBar->hide();
	frameControlPanel->hide();
	setMinimumSize( 602, 676 );
	setMaximumSize( 602, 676 );
	resize( 602, 676 );
	m_pqcontrolpanel->show();
}

void frmDevDynViewer::floatControlPanelToStaticControlPanel() {
	setMinimumSize( 602, 772 );
	setMaximumSize( 602, 772 );
	resize( 602, 772 );
	frameScrollBar->hide();
	frameControlPanel->show();
	m_pqcontrolpanel->hide();
}

void frmDevDynViewer::slotOpen_from_file() {

	QString qsFilePath(
		QFileDialog::getOpenFileName( this, QString( "Open file" )
		, QString::null
		, QString(
			"roi or lng or ddv file (*.roi *.lng *.ddv)" ) ) );
	if ( qsFilePath.isEmpty() ) {
		return;
	}
	int nPoint = qsFilePath.lastIndexOf( QString( "." ) );
	if ( nPoint == -1 ) {
		return;
	}
	QString qsBody = qsFilePath.mid( 0, nPoint );
	QString qsExtension = qsFilePath.mid( nPoint + 1 );
	if ( qsExtension.isEmpty() ) {
		return;
	}

	disableControls();
	if ( m_pcdataset ) {
		m_pcdataset->clear();
	} else {
		m_pcdataset = new CDataset();
	}
	CQDataset_io cqdataset_io;

	std::string sLngPath, sROIPath;
	QFileInfo qfiPath;
	if ( qsExtension == "roi" ) {
		sROIPath = qsFilePath.toStdString();
		qfiPath.setFile( qsBody + ".nlng" );
		if ( qfiPath.isReadable() ) {
			sLngPath = qfiPath.filePath().toStdString();
		} else {
			qfiPath.setFile( qsBody + ".lng" );
			if ( qfiPath.isReadable() ) {
				sLngPath = qfiPath.filePath().toStdString();
			}
		}
	} else if ( qsExtension == "lng" || qsExtension == "nlng" ) {
		sLngPath = qsFilePath.toStdString();
		qfiPath.setFile( qsBody + ".roi" );
		if ( qfiPath.isReadable() ) {
			sROIPath = qfiPath.filePath().toStdString();
		}
	} else if ( qsExtension == "ddv" ) {
		class xmlparser: public QXmlDefaultHandler {
		public:
			bool startDocument() {
				return true;
			};
			bool startElement( const QString &qsNameSpaceURI
				, const QString &qsLocalName, const QString & qsName
				, const QXmlAttributes & qxmlattributes ) {
				qsElementNames.push_back( qsName );
				attributes.push_back( qxmlattributes );
				return true;
			};
			bool endElement( const QString &, const QString &
				, const QString & ) {
				QString qsElementName = qsElementNames.back();
				if ( qsElementName == "file" ) {
					QString qsType
						= attributes.back().value( QString( "type" ) );
					if ( qsType == "roi" ) {
						qsROIFile = qsCData.trimmed();
					} else if ( qsType == "lineage" ) {
						qsLngFile = qsCData.trimmed();
					}
				}
				attributes.pop_back();
				qsElementNames.pop_back();
				return true;
			};
			bool characters( const QString &qsCharacters ) {
				qsCData = qsCharacters;
				return true;
			};

		private:
			QStringList qsElementNames;
			std::vector< QXmlAttributes > attributes;
			QString qsCData;

		public:
			QString qsROIFile;
			QString qsLngFile;
		};

		xmlparser handler;
		QFile qfDDVfile( qsFilePath );
		QXmlInputSource qxmlisSource( &qfDDVfile );
		QXmlSimpleReader reader;
		reader.setContentHandler( &handler );
		reader.setErrorHandler( &handler );
		if ( !reader.parse( &qxmlisSource ) ) {
			showMessage( "load " + qsFilePath + " failed." );
			return;
		}

		qfiPath.setFile( qsFilePath );
		QFileInfo qfiROILng( handler.qsROIFile );
		sROIPath = qfiPath.path().toStdString()
			+ "/" + qfiROILng.fileName().toStdString();
		qfiROILng.setFile( handler.qsLngFile );
		sLngPath = qfiPath.path().toStdString()
			+ "/" + qfiROILng.fileName().toStdString();
	}

	if ( sROIPath.length() && sLngPath.length() ) {
		if ( !cqdataset_io.loadLineageAndROI(
			sLngPath.c_str(), sROIPath.c_str() ) ) {
			showMessage( cqdataset_io.getErrorString().c_str() );
			return;
		}
	} else if ( !sROIPath.length() && sLngPath.length() ) {
		if ( !cqdataset_io.loadLineage( sLngPath.c_str() ) ) {
			showMessage( cqdataset_io.getErrorString().c_str() );
			return;
		}
	} else if ( sROIPath.length() && !sLngPath.length() ) {
		if ( !cqdataset_io.loadROI( sROIPath.c_str() ) ) {
			showMessage( cqdataset_io.getErrorString().c_str() );
			return;
		}
	} else {
		return;
	}

	*m_pcdataset = cqdataset_io;
	m_pcqviewer->setDataset( m_pcdataset );
	controlPanel->setMinTimePoint( m_pcdataset->getMinTimePoint() );
	controlPanel->setMaxTimePoint( m_pcdataset->getMaxTimePoint() );
	m_pfloatControlPanel->setMinTimePoint( m_pcdataset->getMinTimePoint() );
	m_pfloatControlPanel->setMaxTimePoint( m_pcdataset->getMaxTimePoint() );
	timePointChanged( m_pcdataset->getMinTimePoint() );

	enableControls();
	actionClose->setEnabled( true );
	qfiPath.setFile( qsBody );
	controlPanel->setDatasetName( qfiPath.fileName().toStdString() );
	m_pfloatControlPanel->setDatasetName( qfiPath.fileName().toStdString() );
	showMessage( qsBody + " is Loaded.", 10000 );
}

void frmDevDynViewer::slotClose() {
	controlPanel->clear();
	m_pfloatControlPanel->clear();
	m_pcdataset->clear();
	m_pcqviewer->clear();
	disableControls();
	actionClose->setEnabled( false );
	showMessage( "Closed.", 10000 );
}

void frmDevDynViewer::slotViewAllObjects() {
	if ( !m_pcdataset ) {
		return;
	}
	if ( m_pcdataset->isLineageExists() ) {
		actionViewTrace->setChecked( true );
		m_pcqviewer->slotActionViewTrace_toggled( true );
		actionViewCenter->setChecked( true );
		m_pcqviewer->slotActionViewCenter_toggled( true );
	}
	if ( m_pcdataset->isCellNameExists() ) {
		actionViewName->setChecked( true );
		m_pcqviewer->slotActionViewName_toggled( true );
	} else {
		actionViewName->setChecked( false );
		m_pcqviewer->slotActionViewName_toggled( false );
	}
	if ( m_pcdataset->isROIExists() ) {
		actionViewOutline->setChecked( true );
		m_pcqviewer->slotActionViewOutline_toggled( true );
	}
	showMessage( "All objects enabled.", 10000 );
}

void frmDevDynViewer::slotTimePointChanged( int nTimePoint ) {
	if ( m_nTimePoint != nTimePoint ) {
		m_nTimePoint = nTimePoint;
		emit timePointChanged( nTimePoint );
		repaint();
	}
}

void frmDevDynViewer::slotTimePointIncrement() {
	if ( m_pcdataset && m_nTimePoint < m_pcdataset->getMaxTimePoint() ) {
		slotTimePointChanged( m_nTimePoint + 1 );
	}
}

void frmDevDynViewer::slotTimePointDecrement() {
	if ( m_pcdataset && m_nTimePoint > m_pcdataset->getMinTimePoint() ) {
		slotTimePointChanged( m_nTimePoint - 1 );
	}
}

void frmDevDynViewer::about()
{
  QMessageBox::about(this, tr("About Developmental Dynamics Viewer"), tr("Developmental Dynamics Viewer\n2009 RIKEN ASI"));
}

void frmDevDynViewer::disableControls() {
	actionMoveForward->setEnabled( false );
	actionMoveBackward->setEnabled( false );
	actionViewOutline->setEnabled( false );
	actionViewOutline->setChecked( false );
	m_pcqviewer->slotActionViewOutline_toggled( false );
	actionViewName->setEnabled( false );
	actionViewName->setChecked( false );
	m_pcqviewer->slotActionViewName_toggled( false );
	actionViewTrace->setEnabled( false );
	actionViewTrace->setChecked( false );
	m_pcqviewer->slotActionViewTrace_toggled( false );
	actionViewCenter->setEnabled( false );
	actionViewCenter->setChecked( false );
	m_pcqviewer->slotActionViewCenter_toggled( false );
	actionView_all_objects->setEnabled( false );
	actionResetView->setEnabled( false );

	controlPanel->disableControls();
	m_pfloatControlPanel->disableControls();
}

void frmDevDynViewer::enableControls() {
	if ( !m_pcdataset ) {
		return;
	}
	actionMoveForward->setEnabled( true );
	actionMoveBackward->setEnabled( true );
	if ( m_pcdataset->isLineageExists() ) {
		actionViewTrace->setEnabled( true );
		actionViewTrace->setChecked( true );
		m_pcqviewer->slotActionViewTrace_toggled( true );
		actionViewCenter->setEnabled( true );
		actionViewCenter->setChecked( true );
		m_pcqviewer->slotActionViewCenter_toggled( true );
	}
	if ( m_pcdataset->isCellNameExists() ) {
		actionViewName->setEnabled( true );
		actionViewName->setChecked( true );
		m_pcqviewer->slotActionViewName_toggled( true );
	} else {
		actionViewName->setEnabled( false );
		actionViewName->setChecked( false );
		m_pcqviewer->slotActionViewName_toggled( false );
	}
	if ( m_pcdataset->isROIExists() ) {
		actionViewOutline->setEnabled( true );
		actionViewOutline->setChecked( true );
		m_pcqviewer->slotActionViewOutline_toggled( true );
	}
	actionView_all_objects->setEnabled( true );
	actionResetView->setEnabled( true );

	controlPanel->enableControls();
	m_pfloatControlPanel->enableControls();
}

} // namespace ddv
