
#ifndef __H_FRMDEVDYNVIEWER__
#define __H_FRMDEVDYNVIEWER__

#include "ui_frmDevDynViewer.h"
#include "dlgControlPanel.h"
#include "CDataset.h"
#include "CQViewer3D.h"

namespace ddv {

class frmDevDynViewer: public QMainWindow, public Ui::MainWindow {

	Q_OBJECT

public:
	frmDevDynViewer( QWidget *parent = 0 ): QMainWindow( parent )
		, m_pcdataset( NULL ), m_pcqviewer( NULL )
		, m_bControlPanelSwitch( true ) {
		setupUi( this );

		connect( this, SIGNAL( showMessage( const QString &, int ) )
			, statusbar, SLOT( showMessage( const QString &, int ) ) );
		connect( actionOpen_from_file, SIGNAL( activated() )
			, this, SLOT( slotOpen_from_file() ) );
		connect( actionClose, SIGNAL( activated() )
			, this, SLOT( slotClose() ) );
		connect( actionView_all_objects, SIGNAL( activated() )
			, this, SLOT( slotViewAllObjects() ) );
		connect( actionMoveForward, SIGNAL( activated() )
			, this, SLOT( slotTimePointIncrement() ) );
		connect( actionMoveBackward, SIGNAL( activated() )
			, this, SLOT( slotTimePointDecrement() ) );
		connect( actionAbout, SIGNAL( activated() )
			 , this, SLOT( about() ) );


		m_pcqviewer = new CQViewer3D( this );
		vlViewer3D->addWidget( m_pcqviewer );
		vlViewer3D->setSizeConstraint( QLayout::SetFixedSize );
		connect( m_pcqviewer, SIGNAL( showMessage( const QString &, int ) )
			, statusbar, SLOT( showMessage( const QString &, int ) ) );
		connect( actionResetView, SIGNAL( activated() )
			, m_pcqviewer, SLOT( slotResetView() ) );
		connect( m_pcqviewer, SIGNAL( timePointChanged( int ) )
			, this, SLOT( slotTimePointChanged( int ) ) );
		connect( this, SIGNAL( timePointChanged( int ) )
			, m_pcqviewer, SLOT( slotTimePointChanged( int ) ) );

		connect( actionViewOutline, SIGNAL( toggled( bool ) )
			, m_pcqviewer, SLOT( slotActionViewOutline_toggled( bool ) ) );
		connect( actionViewName, SIGNAL( toggled( bool ) )
			, m_pcqviewer, SLOT( slotActionViewName_toggled( bool ) ) );
		connect( actionViewTrace, SIGNAL( toggled( bool ) )
			, m_pcqviewer, SLOT( slotActionViewTrace_toggled( bool ) ) );
		connect( actionViewCenter, SIGNAL( toggled( bool ) )
			, m_pcqviewer, SLOT( slotActionViewCenter_toggled( bool ) ) );

		connect( actionFloatControlpanel, SIGNAL( toggled( bool ) )
			, this, SLOT( slotActionFloatControlPanel_toggled( bool ) ) );

		connect( controlPanel->pbtnFloatControlPanel, SIGNAL( clicked() )
			, this, SLOT( slotControlPanel_switched() ) );

		m_pqcontrolpanel = new dlgControlPanel( this );
		m_pfloatControlPanel = m_pqcontrolpanel->pcontrolpanel;
		connect( m_pqcontrolpanel, SIGNAL( hided() )
			, this, SLOT( slotControlPanel_switched() ) );
		connect( this, SIGNAL( timePointChanged( int ) )
			, m_pfloatControlPanel, SLOT( slotTimePointChanged( int ) ) );
		connect( this, SIGNAL( timePointChanged( int ) )
			, controlPanel, SLOT( slotTimePointChanged( int ) ) );
		connect( m_pfloatControlPanel, SIGNAL( lcdTimePoint_changed( int ) )
			, controlPanel, SLOT( slotTimePointChanged( int ) ) );
		connect( controlPanel, SIGNAL( lcdTimePoint_changed( int ) )
			, m_pfloatControlPanel, SLOT( slotTimePointChanged( int ) ) );
		connect( m_pfloatControlPanel, SIGNAL( lcdTimePoint_changed( int ) )
			, this, SLOT( slotTimePointChanged( int ) ) );
		connect( controlPanel, SIGNAL( lcdTimePoint_changed( int ) )
			, this, SLOT( slotTimePointChanged( int ) ) );
		connect( m_pfloatControlPanel, SIGNAL( pbtnPlayLoop_toggled( bool ) )
			, controlPanel, SLOT( slotPbtnPlayLoop_toggled( bool ) ) );
		connect( controlPanel, SIGNAL( pbtnPlayLoop_toggled( bool ) )
			, m_pfloatControlPanel, SLOT( slotPbtnPlayLoop_toggled( bool ) ) );
		connect( m_pfloatControlPanel, SIGNAL( pbtnPlayOnce_toggled( bool ) )
			, controlPanel, SLOT( slotPbtnPlayOnce_toggled( bool ) ) );
		connect( controlPanel, SIGNAL( pbtnPlayOnce_toggled( bool ) )
			, m_pfloatControlPanel, SLOT( slotPbtnPlayOnce_toggled( bool ) ) );
		connect( m_pfloatControlPanel
			, SIGNAL( spinboxBegin_valueChanged( int ) )
			, controlPanel, SLOT( slotSpinboxBegin_valueChanged( int ) ) );
		connect( controlPanel, SIGNAL( spinboxBegin_valueChanged( int ) )
			, m_pfloatControlPanel
			, SLOT( slotSpinboxBegin_valueChanged( int ) ) );
		connect( m_pfloatControlPanel, SIGNAL( spinboxEnd_valueChanged( int ) )
			, controlPanel, SLOT( slotSpinboxEnd_valueChanged( int ) ) );
		connect( controlPanel, SIGNAL( spinboxEnd_valueChanged( int ) )
			, m_pfloatControlPanel
			, SLOT( slotSpinboxEnd_valueChanged( int ) ) );

		floatControlPanelToStaticControlPanel();
	};
	~frmDevDynViewer() {
		if ( m_pqcontrolpanel ) {
			m_pfloatControlPanel = NULL;
			delete m_pqcontrolpanel;
			m_pqcontrolpanel = NULL;
		}
		if ( m_pcdataset ) {
			delete m_pcdataset;
			m_pcdataset = NULL;
		}
	};

signals:
	void showMessage( const QString &message, int timeout = 0 );
	void timePointChanged( int nTimePoint );

public slots:
	void slotActionFloatControlPanel_toggled( bool bOnOff );
	void slotControlPanel_switched();
	void slotOpen_from_file();
	void slotClose();
	void slotViewAllObjects();
	void slotTimePointChanged( int nTimePoint );
	void slotTimePointIncrement();
	void slotTimePointDecrement();
	void about();

protected:
	void disableControls();
	void enableControls();
	void staticControlPanelToFloatControlPanel();
	void floatControlPanelToStaticControlPanel();

	dlgControlPanel *m_pqcontrolpanel;
	wdtControlPanel *m_pfloatControlPanel;
	CDataset *m_pcdataset;
	CQViewer3D *m_pcqviewer;
	bool m_bControlPanelSwitch;
	int m_nTimePoint;
};

} // namespace ddv

#endif // __H_FRMDEVDYNVIEWER__

