
/*!
	@file	quaternion.cpp
	@brief	provide 3D cell viewing
*/

#ifndef __H_QUATERNION__
#define __H_QUATERNION__

#include <math.h>
#include "float3.h"

namespace ddv {

typedef struct quaternion {
public:
	quaternion(): x( 0.0f ), y( 0.0f ), z( 0.0f ), t( 1.0f ) {};
	quaternion( float3 f3, float ft )
		: x( f3.x ), y( f3.y ), z( f3.z ), t( ft ) {};
	quaternion( float fx, float fy, float fz, float ft )
		: x( fx ), y( fy ), z( fz ), t( ft ) {};

	inline void operator = ( quaternion q ) {
		x = q.x;
		y = q.y;
		z = q.z;
		t = q.t;
	};
	inline quaternion operator * ( quaternion &q ) {
		return quaternion(
			  t * q.x + x * q.t + y * q.z - z * q.y
			, t * q.y + y * q.t + z * q.x - x * q.z
			, t * q.z + z * q.t + x * q.y - y * q.x
			, t * q.t - x * q.x - y * q.y - z * q.z );
	};

	float x;
	float y;
	float z;
	float t;
} quaternion;

inline quaternion quaternionRotationAxis( float3 v, float rad ) {
	rad *= 0.5f;
	float s = sin( rad );
	return quaternion( v * s, cos( rad ) );
}

} // namespace ddv

#endif // __H_QUATERNION__

