
#include "wdtControlPanel.h"

//namespace ocv {

unsigned int wdtControlPanel::playWeight = 300;

void wdtControlPanel::clear() {
	timerStop();
	horizontalScrollBar->setMinimum( 0 );
	horizontalScrollBar->setValue( 0 );
	horizontalScrollBar->setMaximum( 0 );
	setDatasetName( "" );
	setMinTimePoint( 0 );
	setMaxTimePoint( 0 );
	spinboxBegin->setValue( 0 );
	spinboxBegin->setRange( 0, 0 );
	spinboxEnd->setValue( 0 );
	spinboxEnd->setRange( 0, 0 );
	slotTimePointChanged( 0 );
	disableControls();
}

void wdtControlPanel::enableControls() {
	horizontalScrollBar->setEnabled( true );
	pbtnForward->setEnabled( true );
	pbtnForwardEnd->setEnabled( true );
	pbtnBackward->setEnabled( true );
	pbtnBackwardEnd->setEnabled( true );
	pbtnPlayLoop->setEnabled( true );
	pbtnPlayOnce->setEnabled( true );
	spinboxBegin->setEnabled( true );
	spinboxEnd->setEnabled( true );
}

void wdtControlPanel::disableControls() {
	horizontalScrollBar->setEnabled( false );
	pbtnForward->setEnabled( false );
	pbtnForwardEnd->setEnabled( false );
	pbtnBackward->setEnabled( false );
	pbtnBackwardEnd->setEnabled( false );
	pbtnPlayLoop->setEnabled( false );
	pbtnPlayOnce->setEnabled( false );
	spinboxBegin->setEnabled( false );
	spinboxEnd->setEnabled( false );
}

void wdtControlPanel::setMinTimePoint( int nMinTimePoint ) {
	m_nMinTimePoint = nMinTimePoint;
	horizontalScrollBar->setMinimum( nMinTimePoint );
	spinboxBegin->setMinimum( nMinTimePoint );
	spinboxEnd->setMinimum( nMinTimePoint );
	spinboxBegin->setValue( nMinTimePoint );
}

void wdtControlPanel::setMaxTimePoint( int nMaxTimePoint ) {
	m_nMaxTimePoint = nMaxTimePoint;
	horizontalScrollBar->setMaximum( nMaxTimePoint );
	spinboxBegin->setMaximum( nMaxTimePoint );
	spinboxEnd->setMaximum( nMaxTimePoint );
	spinboxEnd->setValue( nMaxTimePoint );
}

void wdtControlPanel::slotTimePointChanged( int nTimePoint ) {
	if ( nTimePoint >= m_nMinTimePoint && nTimePoint <= m_nMaxTimePoint ) {
		lcdTimePoint->display( nTimePoint );
		horizontalScrollBar->setValue( nTimePoint );
	}
}

void wdtControlPanel::setDatasetName( std::string sDatasetName ) {
	lblDataset->setText( sDatasetName.c_str() );
}

void wdtControlPanel::slotSelfScrollBar_valueChanged( int nValue ) {
	lcdTimePoint->display( nValue );
	emit lcdTimePoint_changed( nValue );
}

void wdtControlPanel::slotSelfPbtnForward_clicked() {
	int nTimePoint = lcdTimePoint->intValue();
	if ( nTimePoint < m_nMaxTimePoint ) {
		slotTimePointChanged( ++ nTimePoint );
		emit lcdTimePoint_changed( nTimePoint );
	}
	emit pbtnForward_clicked();
}

void wdtControlPanel::slotSelfPbtnForwardEnd_clicked() {
	if ( lcdTimePoint->intValue() != m_nMaxTimePoint ) {
		slotTimePointChanged( m_nMaxTimePoint );
		emit lcdTimePoint_changed( m_nMaxTimePoint );
	}
	emit pbtnBackwardEnd_clicked();
}

void wdtControlPanel::slotSelfPbtnBackward_clicked() {
	int nTimePoint = lcdTimePoint->intValue();
	if ( nTimePoint > m_nMinTimePoint ) {
		slotTimePointChanged( -- nTimePoint );
		emit lcdTimePoint_changed( nTimePoint );
	}
	emit pbtnBackward_clicked();
}

void wdtControlPanel::slotSelfPbtnBackwardEnd_clicked() {
	if ( lcdTimePoint->intValue() != m_nMinTimePoint ) {
		slotTimePointChanged( m_nMinTimePoint );
		emit lcdTimePoint_changed( m_nMinTimePoint );
	}
	emit pbtnForwardEnd_clicked();
}

void wdtControlPanel::slotSelfPbtnPlayLoop_toggled( bool bOnOff ) {
	if ( m_bSlotPbtnPlayLoopDisable ) {
		return;
	}
	pbtnPlayOnce->setEnabled( !bOnOff );
	if ( bOnOff ) {
		timerStart( true );
	} else {
		timerStop();
	}
	emit pbtnPlayLoop_toggled( bOnOff );
}

void wdtControlPanel::slotPbtnPlayLoop_toggled( bool bOnOff ) {
	m_bSlotPbtnPlayLoopDisable = true;
	pbtnPlayLoop->setChecked( bOnOff );
	m_bSlotPbtnPlayLoopDisable = false;
	pbtnPlayOnce->setEnabled( !bOnOff );
	if ( !bOnOff ) {
		timerStop();
	}
}

void wdtControlPanel::slotSelfPbtnPlayOnce_toggled( bool bOnOff ) {
	if ( m_bSlotPbtnPlayOnceDisable ) {
		return;
	}
	pbtnPlayLoop->setEnabled( !bOnOff );
	if ( bOnOff ) {
		timerStart( false );
	} else {
		timerStop();
	}
	emit pbtnPlayOnce_toggled( bOnOff );
}

void wdtControlPanel::slotPbtnPlayOnce_toggled( bool bOnOff ) {
	m_bSlotPbtnPlayOnceDisable = true;
	pbtnPlayOnce->setChecked( bOnOff );
	m_bSlotPbtnPlayLoopDisable = false;
	pbtnPlayLoop->setEnabled( !bOnOff );
	if ( !bOnOff ) {
		timerStop();
	}
}

void wdtControlPanel::timerStart( bool bLoop ) {
	m_bTimerLoop = bLoop;
	int nTimePoint = spinboxBegin->value();
	slotTimePointChanged( nTimePoint );
	emit lcdTimePoint_changed( nTimePoint );
	m_pqtimer->start( playWeight );
}

void wdtControlPanel::timerStop() {
	m_pqtimer->stop();
}

void wdtControlPanel::slotSelfTimer_timeout() {
	int nTimePoint = lcdTimePoint->intValue() + 1;
	if ( nTimePoint <= spinboxEnd->value() ) {
		slotTimePointChanged( nTimePoint );
		emit lcdTimePoint_changed( nTimePoint );
	} else {
		timerStop();
		if ( m_bTimerLoop ) {
			timerStart( m_bTimerLoop );
		} else {
			pbtnPlayOnce->setChecked( false );
		}
	}
}

void wdtControlPanel::slotSelfSpinboxBegin_valueChanged( int nBeginTimePoint ) {
	spinboxEnd->setMinimum( nBeginTimePoint );
	emit spinboxBegin_valueChanged( nBeginTimePoint );
}

void wdtControlPanel::slotSpinboxBegin_valueChanged( int nBeginTimePoint ) {
	spinboxBegin->setValue( nBeginTimePoint );
	spinboxEnd->setMinimum( nBeginTimePoint );
}

void wdtControlPanel::slotSelfSpinboxEnd_valueChanged( int nEndTimePoint ) {
	spinboxBegin->setMaximum( nEndTimePoint );
	emit spinboxEnd_valueChanged( nEndTimePoint );
}

void wdtControlPanel::slotSpinboxEnd_valueChanged( int nEndTimePoint ) {
	spinboxEnd->setValue( nEndTimePoint );
	spinboxBegin->setMaximum( nEndTimePoint );
}
/*
} // namespace ocv
*/
