
#ifndef __H_WDTCONTROLPANEL__
#define __H_WDTCONTROLPANEL__

#include <QDebug>
#include <QTimer>
#include "ui_wdtControlPanel.h"

class wdtControlPanel: public QWidget, public Ui::ControlPanel {

	Q_OBJECT

public:
	wdtControlPanel( QWidget *parent = 0 )
		: QWidget( parent ), m_nMinTimePoint( 0 ), m_nMaxTimePoint( 0 )
		, m_bSlotPbtnPlayLoopDisable( false )
		, m_bSlotPbtnPlayOnceDisable( false ) {
		setupUi( this );

		connect( horizontalScrollBar, SIGNAL( valueChanged( int ) )
			, this, SLOT( slotSelfScrollBar_valueChanged( int ) ) );

		connect( pbtnForward, SIGNAL( clicked() )
			, this, SLOT( slotSelfPbtnForward_clicked() ) );
		connect( pbtnForwardEnd, SIGNAL( clicked() )
			, this, SLOT( slotSelfPbtnForwardEnd_clicked() ) );
		connect( pbtnBackward, SIGNAL( clicked() )
			, this, SLOT( slotSelfPbtnBackward_clicked() ) );
		connect( pbtnBackwardEnd, SIGNAL( clicked() )
			, this, SLOT( slotSelfPbtnBackwardEnd_clicked() ) );

		connect( pbtnPlayLoop, SIGNAL( toggled( bool ) )
			, this, SLOT( slotSelfPbtnPlayLoop_toggled( bool ) ) );
		connect( pbtnPlayOnce, SIGNAL( toggled( bool ) )
			, this, SLOT( slotSelfPbtnPlayOnce_toggled( bool ) ) );

		connect( spinboxBegin, SIGNAL( valueChanged( int ) )
			, this, SLOT( slotSelfSpinboxBegin_valueChanged( int ) ) );
		connect( spinboxEnd, SIGNAL( valueChanged( int ) )
			, this, SLOT( slotSelfSpinboxEnd_valueChanged( int ) ) );

		m_pqtimer = new QTimer( this );
		connect( m_pqtimer, SIGNAL( timeout() )
			, this, SLOT( slotSelfTimer_timeout() ) );

		clear();
	};
	~wdtControlPanel() {
		if ( m_pqtimer ) {
			delete m_pqtimer;
			m_pqtimer = NULL;
		}
	};

	void clear();
	void enableControls();
	void disableControls();
	void setDatasetName( std::string sDatasetName );
	void setMinTimePoint( int nMinTimePoint );
	void setMaxTimePoint( int nMaxTimePoint );

signals:
	void pbtnForward_clicked();
	void pbtnForwardEnd_clicked();
	void pbtnBackward_clicked();
	void pbtnBackwardEnd_clicked();
	void pbtnPlayLoop_toggled( bool bOnOff );
	void pbtnPlayOnce_toggled( bool bOnOff );
	void lcdTimePoint_changed( int nTimePoint );
	void spinboxBegin_valueChanged( int );
	void spinboxEnd_valueChanged( int );

public slots:
	void slotTimePointChanged( int nTimePoint );
	void slotPbtnPlayLoop_toggled( bool bOnOff );
	void slotPbtnPlayOnce_toggled( bool bOnOff );
	void slotSpinboxBegin_valueChanged( int );
	void slotSpinboxEnd_valueChanged( int );

protected slots:
	void slotSelfScrollBar_valueChanged( int nValue );
	void slotSelfPbtnForward_clicked();
	void slotSelfPbtnForwardEnd_clicked();
	void slotSelfPbtnBackward_clicked();
	void slotSelfPbtnBackwardEnd_clicked();
	void slotSelfPbtnPlayLoop_toggled( bool bOnOff );
	void slotSelfPbtnPlayOnce_toggled( bool bOnOff );
	void slotSelfTimer_timeout();
	void slotSelfSpinboxBegin_valueChanged( int nBeginTimePoint );
	void slotSelfSpinboxEnd_valueChanged( int nEndTimePoint );

protected:
	void timerStart( bool bLoop );
	void timerStop();

	QTimer *m_pqtimer;
	int m_nMinTimePoint;
	int m_nMaxTimePoint;
	bool m_bTimerLoop;
	bool m_bSlotPbtnPlayLoopDisable;
	bool m_bSlotPbtnPlayOnceDisable;

public:
	static unsigned int playWeight;
};

#endif // __H_WDTCONTROLPANEL__
